@@ -3,17 +3,16 @@
 //! This is an internal implementation, not visible to the public API.
 
 #[cfg(feature = "logging")]
-use crate::log::debug;
+use crate::log::trace;
 use crate::{
-    dns_parser::{
-        current_time_millis, split_sub_domain, DnsAddress, DnsPointer, DnsRecordBox, DnsSrv, RRType,
-    },
-    service_info::valid_two_addrs_on_intf,
+    dns_parser::{DnsAddress, DnsPointer, DnsRecordBox, DnsSrv, RRType},
+    service_info::{split_sub_domain, valid_two_addrs_on_intf},
 };
 use if_addrs::Interface;
 use std::{
     collections::{HashMap, HashSet},
     net::IpAddr,
+    time::SystemTime,
 };
 
 /// A cache for all types of DNS records.
@@ -82,7 +81,7 @@ impl DnsCache {
             .filter_map(|(instance, srv_list)| {
                 if let Some(item) = srv_list.first() {
                     if let Some(dns_srv) = item.any().downcast_ref::<DnsSrv>() {
-                        if dns_srv.host == host {
+                        if dns_srv.host() == host {
                             return Some(instance.clone());
                         }
                     }
@@ -102,7 +101,7 @@ impl DnsCache {
                 record
                     .any()
                     .downcast_ref::<DnsAddress>()
-                    .map(|addr| addr.address)
+                    .map(|addr| addr.address())
             })
             .collect()
     }
@@ -132,11 +131,11 @@ impl DnsCache {
             };
 
             // Will verify addresses for the hostname.
-            query_vec.push((srv_record.host.clone(), RRType::A));
-            query_vec.push((srv_record.host.clone(), RRType::AAAA));
+            query_vec.push((srv_record.host().to_string(), RRType::A));
+            query_vec.push((srv_record.host().to_string(), RRType::AAAA));
 
             if let Some(new_expire) = expire_at {
-                if let Some(addrs) = self.addr.get_mut(&srv_record.host) {
+                if let Some(addrs) = self.addr.get_mut(srv_record.host()) {
                     for addr in addrs {
                         addr.set_expire_sooner(new_expire);
                     }
@@ -167,8 +166,9 @@ impl DnsCache {
             let (_, subtype_opt) = split_sub_domain(&entry_name);
             if let Some(subtype) = subtype_opt {
                 if let Some(ptr) = incoming.any().downcast_ref::<DnsPointer>() {
-                    if !self.subtype.contains_key(&ptr.alias) {
-                        self.subtype.insert(ptr.alias.clone(), subtype.to_string());
+                    if !self.subtype.contains_key(ptr.alias()) {
+                        self.subtype
+                            .insert(ptr.alias().to_string(), subtype.to_string());
                     }
                 }
             }
@@ -210,15 +210,18 @@ impl DnsCache {
                     if rtype == RRType::A || rtype == RRType::AAAA {
                         if let Some(addr) = r.any().downcast_ref::<DnsAddress>() {
                             if let Some(addr_b) = incoming.any().downcast_ref::<DnsAddress>() {
-                                should_flush =
-                                    valid_two_addrs_on_intf(&addr.address, &addr_b.address, intf);
+                                should_flush = valid_two_addrs_on_intf(
+                                    &addr.address(),
+                                    &addr_b.address(),
+                                    intf,
+                                );
                             }
                         }
                     }
                 }
 
                 if should_flush {
-                    debug!("FLUSH one record: {:?}", &r);
+                    trace!("FLUSH one record: {:?}", &r);
                     let new_expire = now + 1000;
                     r.set_expire(new_expire);
 
@@ -282,11 +285,11 @@ impl DnsCache {
                 let expired = addr.get_record().is_expired(now);
                 if expired {
                     if let Some(addr_record) = addr.any().downcast_ref::<DnsAddress>() {
-                        debug!("evict expired ADDR: {:?}", addr_record);
+                        trace!("evict expired ADDR: {:?}", addr_record);
                         removed
                             .entry(addr.get_name().to_string())
                             .or_insert_with(HashSet::new)
-                            .insert(addr_record.address);
+                            .insert(addr_record.address());
                     }
                 }
                 !expired
@@ -310,14 +313,14 @@ impl DnsCache {
         for (ty_domain, ptr_records) in self.ptr.iter_mut() {
             for ptr in ptr_records.iter() {
                 if let Some(dns_ptr) = ptr.any().downcast_ref::<DnsPointer>() {
-                    let instance_name = &dns_ptr.alias;
+                    let instance_name = dns_ptr.alias();
 
                     // evict expired SRV records of this instance
                     if let Some(srv_records) = self.srv.get_mut(instance_name) {
                         srv_records.retain(|srv| {
                             let expired = srv.get_record().is_expired(now);
                             if expired {
-                                debug!("expired SRV: {}: {:?}", ty_domain, srv);
+                                trace!("expired SRV: {}: {:?}", ty_domain, srv);
                                 expired_instances
                                     .entry(ty_domain.to_string())
                                     .or_insert_with(HashSet::new)
@@ -339,11 +342,11 @@ impl DnsCache {
                 let expired = x.get_record().is_expired(now);
                 if expired {
                     if let Some(dns_ptr) = x.any().downcast_ref::<DnsPointer>() {
-                        debug!("expired PTR: domain:{ty_domain} record: {:?}", dns_ptr);
+                        trace!("expired PTR: domain:{ty_domain} record: {:?}", dns_ptr);
                         expired_instances
                             .entry(ty_domain.to_string())
                             .or_insert_with(HashSet::new)
-                            .insert(dns_ptr.alias.clone());
+                            .insert(dns_ptr.alias().to_string());
                     }
                 }
                 !expired
@@ -372,7 +375,7 @@ impl DnsCache {
     pub(crate) fn refresh_due_srv(&mut self, ty_domain: &str) -> (HashSet<String>, HashSet<u64>) {
         let now = current_time_millis();
 
-        let instances: Vec<&String> = self
+        let instances: Vec<_> = self
             .ptr
             .get(ty_domain)
             .into_iter()
@@ -382,7 +385,7 @@ impl DnsCache {
                 record
                     .any()
                     .downcast_ref::<DnsPointer>()
-                    .map(|ptr| &ptr.alias)
+                    .map(|ptr| ptr.alias())
             })
             .collect();
 
@@ -399,7 +402,7 @@ impl DnsCache {
                 .collect();
 
             if !refresh_timers.is_empty() {
-                refresh_due.insert(instance.clone());
+                refresh_due.insert(instance.to_string());
                 new_timers.extend(refresh_timers);
             }
         }
@@ -412,7 +415,7 @@ impl DnsCache {
     pub(crate) fn refresh_due_hosts(&mut self, ty_domain: &str) -> (HashSet<String>, HashSet<u64>) {
         let now = current_time_millis();
 
-        let instances: Vec<&String> = self
+        let instances: Vec<_> = self
             .ptr
             .get(ty_domain)
             .into_iter()
@@ -422,7 +425,7 @@ impl DnsCache {
                 record
                     .any()
                     .downcast_ref::<DnsPointer>()
-                    .map(|ptr| &ptr.alias)
+                    .map(|ptr| ptr.alias())
             })
             .collect();
 
@@ -438,7 +441,7 @@ impl DnsCache {
                     record
                         .any()
                         .downcast_ref::<DnsSrv>()
-                        .map(|srv| srv.host.clone())
+                        .map(|srv| srv.host().to_string())
                 })
                 .collect();
 
@@ -485,7 +488,7 @@ impl DnsCache {
 
                 Some((
                     hostname.to_owned(),
-                    record.any().downcast_ref::<DnsAddress>().unwrap().address,
+                    record.any().downcast_ref::<DnsAddress>().unwrap().address(),
                 ))
             })
             .collect()
@@ -526,3 +529,11 @@ impl DnsCache {
             .collect()
     }
 }
+
+/// Returns UNIX time in millis
+pub(crate) fn current_time_millis() -> u64 {
+    SystemTime::now()
+        .duration_since(SystemTime::UNIX_EPOCH)
+        .expect("failed to get current UNIX time")
+        .as_millis() as u64
+}
