@@ -1,9 +1,9 @@
 //! Define `ServiceInfo` to represent a service and its operations.
 
 #[cfg(feature = "logging")]
-use crate::log::{error, info};
+use crate::log::debug;
 use crate::{
-    dns_parser::{split_sub_domain, DnsRecordBox, DnsRecordExt, DnsSrv, RRType},
+    dns_parser::{DnsRecordBox, DnsRecordExt, DnsSrv, RRType},
     Error, Result,
 };
 use if_addrs::{IfAddr, Interface};
@@ -715,7 +715,7 @@ pub(crate) fn decode_txt(txt: &[u8]) -> Vec<TxtProperty> {
 
         let offset_end = offset + length;
         if offset_end > txt.len() {
-            error!("DNS TXT record contains invalid data: Size given for property would be out of range. (offset={}, length={}, offset_end={}, record length={})", offset, length, offset_end, txt.len());
+            debug!("DNS TXT record contains invalid data: Size given for property would be out of range. (offset={}, length={}, offset_end={}, record length={})", offset, length, offset_end, txt.len());
             break; // Skipping the rest of the record content, as the size for this property would already be out of range.
         }
         let kv_bytes = &txt[offset..offset_end];
@@ -734,7 +734,7 @@ pub(crate) fn decode_txt(txt: &[u8]) -> Vec<TxtProperty> {
                     val: v,
                 });
             }
-            Err(e) => error!("failed to convert to String from key: {}", e),
+            Err(e) => debug!("failed to convert to String from key: {}", e),
         }
 
         offset += length;
@@ -936,7 +936,7 @@ impl DnsRegistry {
         if let Some(active_records) = self.active.get(answer.get_name()) {
             for record in active_records.iter() {
                 if answer.matches(record.as_ref()) {
-                    info!(
+                    debug!(
                         "found active record {} {}",
                         answer.get_type(),
                         answer.get_name(),
@@ -950,7 +950,7 @@ impl DnsRegistry {
             .probing
             .entry(answer.get_name().to_string())
             .or_insert_with(|| {
-                info!("new probe of {}", answer.get_name());
+                debug!("new probe of {}", answer.get_name());
                 Probe::new(start_time)
             });
 
@@ -958,7 +958,7 @@ impl DnsRegistry {
 
         for record in probe.records.iter() {
             if answer.matches(record.as_ref()) {
-                info!(
+                debug!(
                     "found existing record {} in probe of '{}'",
                     answer.get_type(),
                     answer.get_name(),
@@ -968,7 +968,7 @@ impl DnsRegistry {
             }
         }
 
-        info!(
+        debug!(
             "insert record {} into probe of {}",
             answer.get_type(),
             answer.get_name(),
@@ -995,9 +995,9 @@ impl DnsRegistry {
             probe.records.retain(|record| {
                 if record.get_type() == RRType::SRV {
                     if let Some(srv) = record.any().downcast_ref::<DnsSrv>() {
-                        if srv.host == original {
+                        if srv.host() == original {
                             let mut new_record = srv.clone();
-                            new_record.host = new_name.to_string();
+                            new_record.set_host(new_name.to_string());
                             found_records.push(new_record);
                             return false;
                         }
@@ -1011,9 +1011,9 @@ impl DnsRegistry {
             records.retain(|record| {
                 if record.get_type() == RRType::SRV {
                     if let Some(srv) = record.any().downcast_ref::<DnsSrv>() {
-                        if srv.host == original {
+                        if srv.host() == original {
                             let mut new_record = srv.clone();
-                            new_record.host = new_name.to_string();
+                            new_record.set_host(new_name.to_string());
                             found_records.push(new_record);
                             return false;
                         }
@@ -1039,7 +1039,7 @@ impl DnsRegistry {
                 }
             };
 
-            info!(
+            debug!(
                 "insert record {} with new hostname {new_name} into probe for: {}",
                 record.get_type(),
                 record.get_name()
@@ -1051,6 +1051,15 @@ impl DnsRegistry {
     }
 }
 
+/// Returns a tuple of (service_type_domain, optional_sub_domain)
+pub(crate) fn split_sub_domain(domain: &str) -> (&str, Option<&str>) {
+    if let Some((_, ty_domain)) = domain.rsplit_once("._sub.") {
+        (ty_domain, Some(domain))
+    } else {
+        (domain, None)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::{
