@@ -476,6 +476,7 @@ fn test_into_txt_properties() {
 }
 
 /// Test enabling an interface using its name, for example "en0".
+/// Also tests an instance name with Upper Case.
 #[test]
 fn service_with_named_interface_only() {
     // Create a daemon
@@ -491,7 +492,7 @@ fn service_with_named_interface_only() {
     let port = 5202;
     let my_service = ServiceInfo::new(
         my_ty_domain,
-        "my_instance",
+        "UpperCaseInstance",
         host_name,
         host_ipv4,
         port,
@@ -838,8 +839,20 @@ fn test_service_name_check() {
     assert!(result.is_ok());
 
     // Verify that the service was published successfully.
-    let event = monitor.recv_timeout(Duration::from_millis(500)).unwrap();
-    assert!(matches!(event, DaemonEvent::Announce(_, _)));
+    let mut published = false;
+    let publish_timeout = 1200;
+    while let Ok(event) = monitor.recv_timeout(Duration::from_millis(publish_timeout)) {
+        match event {
+            DaemonEvent::Announce(_, _) => {
+                published = true;
+                break;
+            }
+            other => {
+                println!("other daemon events: {:?}", other);
+            }
+        }
+    }
+    assert!(published);
 
     // Check for the internal upper limit of service name length max.
     let r = server_daemon.set_service_name_len_max(31);
@@ -933,11 +946,20 @@ fn instance_name_two_dots() {
     assert!(result.is_ok());
 
     // Verify that the service was published successfully.
-    let publish_timeout = 1000;
-    let event = monitor
-        .recv_timeout(Duration::from_millis(publish_timeout))
-        .unwrap();
-    assert!(matches!(event, DaemonEvent::Announce(_, _)));
+    let mut published = false;
+    let publish_timeout = 1200;
+    while let Ok(event) = monitor.recv_timeout(Duration::from_millis(publish_timeout)) {
+        match event {
+            DaemonEvent::Announce(_, _) => {
+                published = true;
+                break;
+            }
+            other => {
+                println!("other daemon events: {:?}", other);
+            }
+        }
+    }
+    assert!(published);
 
     // Browse the service.
     let receiver = server_daemon.browse(service_type).unwrap();
@@ -1546,9 +1568,12 @@ fn test_name_conflict_resolution() {
                 service_names.insert(info.get_fullname().to_string());
 
                 // Find and verify name conflict resolution.
-
                 if info.get_fullname().contains("(2)") {
                     assert_eq!(info.get_hostname(), "conflict_host-2.local.");
+                }
+
+                // Stop the wait if both are resolved.
+                if service_names.len() == 2 {
                     break;
                 }
             }
@@ -1854,6 +1879,173 @@ fn test_verify_srv() {
     assert!(service_removal);
 }
 
+#[test]
+fn test_multicast_loop_v4() {
+    let ty_domain = "_loop_v4._udp.local.";
+    let now = SystemTime::now()
+        .duration_since(SystemTime::UNIX_EPOCH)
+        .unwrap();
+    let instance_name = now.as_micros().to_string(); // Create a unique name.
+    let host_name = "loop_v4_host.local.";
+    let port = 5200;
+
+    // Register the first service.
+    let server = ServiceDaemon::new().expect("failed to start server");
+    server.set_multicast_loop_v4(false).unwrap();
+
+    // Get a single IPv4 address
+    let ip_addr1 = my_ip_interfaces()
+        .iter()
+        .find(|iface| iface.ip().is_ipv4())
+        .map(|iface| iface.ip())
+        .unwrap();
+
+    // Publish the service on server
+    let service1 = ServiceInfo::new(ty_domain, &instance_name, host_name, &ip_addr1, port, None)
+        .expect("valid service info");
+    server
+        .register(service1)
+        .expect("Failed to register service1");
+
+    // wait for the service announced.
+    sleep(Duration::from_secs(1));
+
+    // start a client i.e. querier.
+    let mut resolved = false;
+    let client = ServiceDaemon::new().expect("failed to create mdns client");
+
+    // For Windows, IP_MULTICAST_LOOP option works only on the receive path.
+    client.set_multicast_loop_v4(false).unwrap();
+
+    let receiver = client.browse(ty_domain).unwrap();
+
+    let timeout = Duration::from_secs(2);
+    while let Ok(event) = receiver.recv_timeout(timeout) {
+        match event {
+            ServiceEvent::ServiceResolved(info) => {
+                println!(
+                    "Resolved a service: {} host {} IP {:?}",
+                    info.get_fullname(),
+                    info.get_hostname(),
+                    info.get_addresses_v4()
+                );
+                resolved = true;
+                break;
+            }
+            _ => {}
+        }
+    }
+
+    assert_eq!(resolved, false);
+
+    // enable loopback and try again.
+    server.set_multicast_loop_v4(true).unwrap();
+    client.set_multicast_loop_v4(true).unwrap();
+    let receiver = client.browse(ty_domain).unwrap();
+
+    while let Ok(event) = receiver.recv_timeout(timeout) {
+        match event {
+            ServiceEvent::ServiceResolved(info) => {
+                println!(
+                    "Resolved a service: {} host {} IP {:?}",
+                    info.get_fullname(),
+                    info.get_hostname(),
+                    info.get_addresses_v4()
+                );
+                resolved = true;
+                break;
+            }
+            _ => {}
+        }
+    }
+
+    assert!(resolved);
+}
+
+#[test]
+fn test_multicast_loop_v6() {
+    let ty_domain = "_loop_v6._udp.local.";
+    let now = SystemTime::now()
+        .duration_since(SystemTime::UNIX_EPOCH)
+        .unwrap();
+    let instance_name = now.as_micros().to_string(); // Create a unique name.
+    let host_name = "loop_v6_host.local.";
+    let port = 5200;
+
+    // Register the first service.
+    let server = ServiceDaemon::new().expect("failed to start server");
+    server.set_multicast_loop_v6(false).unwrap();
+
+    // Get a single IPv4 address
+    let ip_addr1 = my_ip_interfaces()
+        .iter()
+        .find(|iface| iface.ip().is_ipv6())
+        .map(|iface| iface.ip())
+        .unwrap();
+
+    // Publish the service on server
+    let service1 = ServiceInfo::new(ty_domain, &instance_name, host_name, &ip_addr1, port, None)
+        .expect("valid service info");
+    server
+        .register(service1)
+        .expect("Failed to register service1");
+
+    // wait for the service announced.
+    sleep(Duration::from_secs(1));
+
+    // start a client i.e. querier.
+    let mut resolved = false;
+    let client = ServiceDaemon::new().expect("failed to create mdns client");
+
+    // For Windows, IP_MULTICAST_LOOP option works only on the receive path.
+    client.set_multicast_loop_v6(false).unwrap();
+
+    let receiver = client.browse(ty_domain).unwrap();
+
+    let timeout = Duration::from_secs(2);
+    while let Ok(event) = receiver.recv_timeout(timeout) {
+        match event {
+            ServiceEvent::ServiceResolved(info) => {
+                println!(
+                    "Resolved a service: {} host {} IP {:?}",
+                    info.get_fullname(),
+                    info.get_hostname(),
+                    info.get_addresses()
+                );
+                resolved = true;
+                break;
+            }
+            _ => {}
+        }
+    }
+
+    assert_eq!(resolved, false);
+
+    // enable loopback and try again.
+    server.set_multicast_loop_v6(true).unwrap();
+    client.set_multicast_loop_v6(true).unwrap();
+
+    let receiver = client.browse(ty_domain).unwrap();
+
+    while let Ok(event) = receiver.recv_timeout(timeout) {
+        match event {
+            ServiceEvent::ServiceResolved(info) => {
+                println!(
+                    "Resolved a service: {} host {} IP {:?}",
+                    info.get_fullname(),
+                    info.get_hostname(),
+                    info.get_addresses()
+                );
+                resolved = true;
+                break;
+            }
+            _ => {}
+        }
+    }
+
+    assert!(resolved);
+}
+
 /// A helper function to include a timestamp for println.
 fn timed_println(msg: String) {
     let now = SystemTime::now();
